@@ -1,4 +1,4 @@
-# $Id: estimable.R,v 1.9 2005/12/04 06:12:22 warnes Exp $
+# $Id: estimable.R 964 2006-06-05 20:59:50Z nj7w $
 
 estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
                            show.beta0)
@@ -9,12 +9,11 @@ estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
     }
   else if(is.list(cm))
     {
-      cm <- t(sapply(cm,
-                     .to.est, obj=obj)) 
-    }
+      cm <- matrix(.to.est(obj, cm), nrow=1) ################### changed 
+    }                                        ################### it seems that the names are lost with the way it used to be...
   else if(is.vector(cm))
     {
-      cm <- matrix(.to.est(obj, cm),nrow=1)
+      cm <- matrix(.to.est(obj, cm), nrow=1)
     }
   else
     {
@@ -43,23 +42,7 @@ estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
     }
   else
     {
-      if ("lmer" %in% class(obj)) {
-        stat.name <- "t.stat"
-        cf <- as.matrix(fixef(obj))
-        vcv <- vcov(obj)
-        tmp <- cm
-        tmp[tmp==0] <- NA
-        df.all <- t(abs(t(tmp) * getFixDF(obj)))
-        df <- apply(df.all, 1, min, na.rm=TRUE)
-        problem <- apply(df.all != df, 1, any, na.rm=TRUE)
-        if (any(problem))
-          warning(paste("Degrees of fredom vary among parameters used to ",
-                        "constrct linear contrast(s): ",
-                        paste((1:nrow(tmp))[problem], collapse=", "),
-                        ". Using the smallest df among the set of parameters.",
-                        sep=""))
-      }
-      else if ("lme" %in% class(obj)) {
+      if ("lme" %in% class(obj)) {
         stat.name <- "t.stat"
         cf <- summary(obj)$tTable
         rho <- summary(obj)$cor
@@ -113,7 +96,7 @@ estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
         }
       else
         {
-          stop("obj must be of class 'lm', 'glm', 'aov', 'lme', 'lmer', 'gee', 'geese' or 'nlme'")
+          stop("obj must be of class 'lm', 'glm', 'aov', 'lme', 'gee', 'geese' or 'nlme'")
         }
       if (is.null(cm))
         cm <- diag(dim(cf)[1])
@@ -136,7 +119,7 @@ estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
              },
              X2.stat={
                prob <- 1 - pchisq((ct.diff/vc)^2, df=1)
-             })
+             })                                                      ##################
 
       if (stat.name=="X2.stat")
         {
@@ -153,17 +136,19 @@ estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
           dimnames(retval) <- list(rn, c("beta0", "Estimate", "Std. Error",
                                          "t value", "DF", "Pr(>|t|)"))
         }
-
+      
       if (!is.null(conf.int))
         {
           if (conf.int <=0 || conf.int >=1)
             stop("conf.int should be between 0 and 1. Usual values are 0.95, 0.90")
           alpha <- 1 - conf.int
-          switch(stat.name, t.stat={
-            quant <- qt(1 - alpha/2, df)
-          }, X2.stat={
-            quant <- qt(1 - alpha/2, 100)
-          })
+          switch(stat.name,
+                 t.stat={
+                   quant <- qt(1 - alpha/2, df)
+                 },
+                 X2.stat={
+                   quant <- qt(1 - alpha/2, 100)
+                 })
           nm <- c(colnames(retval), "Lower.CI", "Upper.CI")
           retval <- cbind(retval, lower=ct.diff - vc * quant, upper=ct.diff +
                           vc * quant)
@@ -212,15 +197,65 @@ estimable <- function (obj, cm, beta0, conf.int=NULL, joint.test=FALSE,
     print(as.data.frame(retval))
 }
 
-
-## this is how the DF are caclulated in the Matrix package (for lmer.summary objects)
-## it seems that this is not entirely correct, but will hopfully be improved upon shortly
-## see lmer.R from the Matrix package version 0.99-1
-
-getFixDF <- function(obj)
+estimable.lmer <- function (obj, cm, beta0, conf.int=NULL,
+                            show.beta0, sim.lmer=TRUE, n.sim=1000)
 {
-          nc <- obj@nc[-seq(along = obj@Omega)]
-          p <- abs(nc[1]) - 1
-          n <- nc[2]
-          rep(n-p, p)
+  if (is.matrix(cm) || is.data.frame(cm))
+    {
+      cm <- t(apply(cm, 1, .to.est, obj=obj)) 
+    }
+  else if(is.list(cm))
+    {
+      cm <- matrix(.to.est(obj, cm), nrow=1)
+    }                                       
+  else if(is.vector(cm))
+    {
+      cm <- matrix(.to.est(obj, cm), nrow=1)
+    }
+  else
+    {
+      stop("`cm' argument must be of type vector, list, or matrix.")
+    }
+
+  if(missing(show.beta0))
+    {
+      if(!missing(beta0))
+        show.beta0=TRUE
+      else
+        show.beta0=FALSE
+    }
+
+
+  if (missing(beta0))
+    {
+      beta0 = rep(0, ifelse(is.null(nrow(cm)), 1, nrow(cm)))
+
+    }
+
+    if ("lmer" %in% class(obj)) {                                      
+      if(!require(Matrix, quietly=TRUE))                               
+        stop("Matrix package required for lmer objects")               
+                                                                       
+      if(sim.lmer)                                                     
+        return(est.lmer(obj=obj, cm=cm, beta0=beta0, conf.int=conf.int,
+                        show.beta0=show.beta0, n.sim=n.sim))           
+                                                                       
+      stat.name <- "lmer"                                              
+      cf <- as.matrix(fixef(obj))                                      
+      vcv <- as.matrix(vcov(obj))                                      
+      df <- NA                                                         
+    }
+    else {
+      stop("obj is not of class lmer")
+    }
+
+    retval <- cbind(hyp=beta0, est=ct, stderr=vc, "t value"=ct.diff/vc)
+    dimnames(retval) <- list(rn, c("beta0", "Estimate", "Std. Error",  
+                                   "t value"))
+
+    rownames(retval) <- make.unique(rownames(retval))
+    retval <- as.data.frame(retval)
+    if(!show.beta0) retval$beta0 <- NULL
+    return(retval)
+
 }
